@@ -1,31 +1,47 @@
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
+"""
+!/usr/bin/env python3
+-*- coding: utf-8 -*-
 
-# ----------------------------------------------------------------------
-# Copyright 2021 CFS Engineering
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#     http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-# ----------------------------------------------------------------------
+----------------------------------------------------------------------
+Copyright 2021 CFS Engineering
 
-# Author: Aidan Jungo
+Licensed under the Apache License, Version 2.0 (the "License");
+you may not use this file except in compliance with the License.
+You may obtain a copy of the License at
+
+    http://www.apache.org/licenses/LICENSE-2.0
+
+Unless required by applicable law or agreed to in writing, software
+distributed under the License is distributed on an "AS IS" BASIS,
+WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+See the License for the specific language governing permissions and
+limitations under the License.
+----------------------------------------------------------------------
+
+Author: Aidan Jungo
+
+"""
 
 import numpy as np
 
-import tigl3.configuration
-import tixi3.tixi3wrapper as tixi3wrapper
-import tigl3.tigl3wrapper as tigl3wrapper
-from tixi3.tixi3wrapper import Tixi3Exception
-from tigl3.tigl3wrapper import Tigl3Exception
+try:
+    from tixi3 import tixi3wrapper
+    from tixi3.tixi3wrapper import Tixi3Exception
+
+except ImportError:
+    TIXI_INSTALLED = False
+else:
+    TIXI_INSTALLED = True
+
+try:
+    import tigl3.configuration
+    from tigl3 import tigl3wrapper
+    # from tigl3.tigl3wrapper import Tigl3Exception
+
+except ImportError:
+    TIGL_INSTALLED = False
+else:
+    TIGL_INSTALLED = True
 
 
 def open_tixi(cpacs_path):
@@ -42,6 +58,14 @@ def open_tixi(cpacs_path):
         tixi_handle (handles): TIXI Handle of the CPACS file
     """
 
+    if not TIXI_INSTALLED:
+        err_msg = """
+        Unable to import Tixi. Please make sure Tixi is accessible to Python.
+        Please refer to the documentation to check supported versions of Tixi.
+        """
+        print(err_msg)
+        raise ModuleNotFoundError(err_msg)
+
     tixi_handle = tixi3wrapper.Tixi3()
     tixi_handle.open(cpacs_path)
 
@@ -64,7 +88,15 @@ def open_tigl(tixi_handle):
         tigl_handle (handles): TIGL Handle of the CPACS file
     """
 
-    # Get model uid, requierd to open TiGL handle in case there is also a rotorcraft in the CPACS file
+    if not TIGL_INSTALLED:
+        err_msg = """
+        Unable to import Tigl. Please make sure Tigl is accessible to Python.
+        Please refer to the documentation to check supported versions of Tigl.
+        """
+        print(err_msg)
+        raise ModuleNotFoundError(err_msg)
+
+    # Get model uid to open TiGL handle (in case there is also a rotorcraft in the CPACS file)
     model_xpath = '/cpacs/vehicles/aircraft/model'
     if tixi_handle.checkAttribute(model_xpath, 'uID'):
         model_uid = tixi_handle.getTextAttribute(model_xpath, 'uID')
@@ -81,63 +113,20 @@ def open_tigl(tixi_handle):
 
 def get_tigl_configuration(tigl):
     """ Get the TiGL aircraft configuration manager. """
-    
+
     # Get the configuration manager
-    mgr =  tigl3.configuration.CCPACSConfigurationManager_get_instance()
+    mgr = tigl3.configuration.CCPACSConfigurationManager_get_instance()
     aircraft = mgr.get_configuration(tigl._handle.value)
-    
+
     return aircraft
 
 
-def get_value(tixi, xpath):
-    """ Check first if the the xpath exist and a value is store
-    at this place. Then, it gets and returns this value. If the value or the
-    xpath does not exist it raise an error and return 'None'.
-
-    Source :
-        * TIXI functions: http://tixi.sourceforge.net/Doc/index.html
-
-    Args:
-        tixi_handle (handles): TIXI Handle of the CPACS file
-        xpath (str): xpath of the value to get
-
-    Returns:
-         value (float or str): Value found at xpath
-    """
-
-    # Try to get the a value at xpath
-    try:
-        value = tixi.getTextElement(xpath)
-    except:
-        value = None
-
-    if value:
-        try: # check if it is a 'float'
-            is_float = isinstance(float(value), float)
-            value = float(value)
-        except:
-            pass
-    else:
-        # check if the path exist
-        if tixi.checkElement(xpath):
-            raise ValueError('No value has been found at ' + xpath)
-        else:
-            raise ValueError(xpath + ' cannot be found in the CPACS file')
-
-    # Special return for boolean
-    if value == 'True':
-        return True
-    elif value == 'False':
-        return False
-
-    return value
-
 def copy_branch(tixi, xpath_from, xpath_to):
     """ Function to copy a CPACS branch.
 
     Function 'copy_branch' copy the branch (with sub-branches) from
     'xpath_from' to 'xpath_to' by using recursion. The new branch should
-    be identical (uiD, attribute, etc). 
+    be identical (uiD, attribute, etc).
 
     Source :
         * TIXI functions: http://tixi.sourceforge.net/Doc/index.html
@@ -159,12 +148,10 @@ def copy_branch(tixi, xpath_from, xpath_to):
     child_nb = tixi.getNumberOfChilds(xpath_from)
 
     if child_nb:
-        xpath_to_split = xpath_to.split("/")
-        xpath_to_parent = '/'.join(str(m) for m in xpath_to_split[:-1])
 
         child_list = []
         for i in range(child_nb):
-            child_list.append(tixi.getChildNodeName(xpath_from, i+1))
+            child_list.append(tixi.getChildNodeName(xpath_from, i + 1))
 
         # If it is a text Element --> no child
         if "#" in child_list[0]:
@@ -174,14 +161,11 @@ def copy_branch(tixi, xpath_from, xpath_to):
         else:
             # If child are named child (e.g. wings/wing)
             if all(x == child_list[0] for x in child_list):
-                namedchild_nb = tixi.getNamedChildrenCount(xpath_from,
-                                                           child_list[0])
+                namedchild_nb = tixi.getNamedChildrenCount(xpath_from, child_list[0])
 
                 for i in range(namedchild_nb):
-                    new_xpath_from = xpath_from + "/" + child_list[0] \
-                                     + '[' + str(i+1) + ']'
-                    new_xpath_to = xpath_to + "/" + child_list[0] \
-                                   + '[' + str(i+1) + ']'
+                    new_xpath_from = xpath_from + "/" + child_list[0] + '[' + str(i + 1) + ']'
+                    new_xpath_to = xpath_to + "/" + child_list[0] + '[' + str(i + 1) + ']'
                     tixi.createElement(xpath_to, child_list[0])
 
                     # Call the function itself for recursion
@@ -207,9 +191,10 @@ def copy_branch(tixi, xpath_from, xpath_to):
                 attrib_text = tixi.getTextAttribute(xpath_from, attrib_name)
                 tixi.addTextAttribute(xpath_to, attrib_name, attrib_text)
                 attrib_index = attrib_index + 1
-            except:
+            except Tixi3Exception:
                 last_attrib = 1
 
+
 def get_uid(tixi, xpath):
     """ Function to get uID from a specific XPath.
 
@@ -226,7 +211,6 @@ def get_uid(tixi, xpath):
         uid (str): uid to add at xpath
     """
 
-
     if not tixi.checkElement(xpath):
         raise ValueError(xpath + ' XPath does not exist!')
 
@@ -264,10 +248,13 @@ def add_uid(tixi, xpath, uid):
             uid_new = uid + str(i)
             print('UID already existing changed to: ' + uid_new)
 
-def get_value_or_default(tixi,xpath,default_value):
-    """ Do the same than the function 'get_value'
-    but if no value is found at this xpath it returns the default value and add
-    it at the xpath. If the xpath does not exist, it is created.
+
+def get_value(tixi, xpath):
+    """ Check first if the the xpath exist and that a value is stored
+    at this place. Returns this value. It returns a:
+    - float value if the value can be read as a float
+    - boolean if the value is 'True'/'False',
+    - otherwise a string
 
     Source :
         * TIXI functions: http://tixi.sourceforge.net/Doc/index.html
@@ -275,59 +262,96 @@ def get_value_or_default(tixi,xpath,default_value):
     Args:
         tixi_handle (handles): TIXI Handle of the CPACS file
         xpath (str): xpath of the value to get
-        default_value (str, float or int): Default value
+
+    Returns:
+         value (float, bool, str): Value found at xpath
+    """
+
+    if not tixi.checkElement(xpath):
+        raise ValueError(f'{xpath} cannot be found in the CPACS file')
+
+    value = tixi.getTextElement(xpath)
+
+    if not value:
+        raise ValueError(f'No value has been found at {xpath}')
+
+    # Check if the value should be return as boolean
+    if value == 'True':
+        return True
+    elif value == 'False':
+        return False
+
+    # Check if the value should be return as float
+    try:
+        float(value)
+        return float(value)
+    except ValueError:
+        pass
+
+    # Otherwise, return the value as a string
+    return value
+
+
+def get_value_or_default(tixi, xpath, default_value):
+    """ Do the same than the function 'get_value' but if no value is found
+    at the xpath it returns the default value and add it in the CPACS file
+    at the xpath. If the xpath does not exist, it is created.
+
+    Source :
+        * TIXI functions: http://tixi.sourceforge.net/Doc/index.html
+
+    Args:
+        tixi (handles): TIXI Handle of the CPACS file
+        xpath (str): xpath of the value to get
+        default_value (str, bool, float or int): Default value
 
     Returns:
         tixi (handles): Modified TIXI Handle (with added default value)
-        value (str, float or int): Value found at xpath
+        default_value (str, bool, float or int): Value found at xpath
     """
 
-    value = None
+    # Try to get the value in the CPACS file
     try:
-        value = get_value(tixi, xpath)
-    except:
+        return get_value(tixi, xpath)
+    except ValueError:
         pass
 
-    if value is None:
+    # Prepare to write the default value in the CPACS file
+    xpath_parent = get_xpath_parent(xpath, 1)
+    create_branch(tixi, xpath_parent, False)
+    value_name = xpath.split("/")[-1]
+
+    is_bool = False
+    is_float = False
+
+    # Check if the default_value could be float
+    try:
+        value = float(default_value)
+        is_float = True
+    except ValueError:
+        pass
+
+    # Copy the default value (in case it is a string)
+    if not is_float:
         value = default_value
 
-        xpath_parent = '/'.join(str(m) for m in xpath.split("/")[:-1])
-        value_name = xpath.split("/")[-1]
-        create_branch(tixi,xpath_parent,False)
-
-        try:
-            is_int = isinstance(float(default_value), int)
-        except:
-            is_int = False
-
-        try:
-            is_float = isinstance(float(default_value), float)
-        except:
-            is_float = False
-
-        try:
-            is_bool = isinstance(default_value, bool)
-        except:
-            is_bool = False
-
-            
-        if is_bool:
-           tixi.addTextElement(xpath_parent,value_name,str(value))
-        elif is_float or is_int:
-            value = float(default_value)
-            tixi.addDoubleElement(xpath_parent,value_name,value,'%g')
+    # Check if the value should be return as boolean
+    if isinstance(default_value, bool):
+        is_bool = True
+        if default_value:
+            value = True
         else:
-            value = str(value)
-            tixi.addTextElement(xpath_parent,value_name,value)
-    else:
-        # Special return for boolean
-        if value == 'True':
-            return True
-        elif value == 'False':
-            return False
-        elif isinstance(value,bool):
-            return value
+            value = False
 
+    # Write the default value in the CPACS file as a float
+    if is_float and not is_bool:
+        tixi.addDoubleElement(xpath_parent, value_name, value, '%g')
+        return value
+
+    # Write the default value in the CPACS file as a string (also for booleans)
+    tixi.addTextElement(xpath_parent, value_name, str(value))
+
+    # Return the value as a string
     return value
 
 
@@ -350,15 +374,16 @@ def get_float_vector(tixi, xpath):
 
     if float_vector_str.endswith(';'):
         float_vector_str = float_vector_str[:-1]
-    
+
     float_vector_list = float_vector_str.split(';')
-    float_vector = [np.nan if (elem == 'nan' or elem == 'NaN') else float(elem) for elem in float_vector_list]
+    float_vector = [np.nan if elem in ('nan', 'NaN')
+                    else float(elem) for elem in float_vector_list]
 
     return float_vector
 
 
 def add_float_vector(tixi, xpath, vector):
-    """ Add a vector (composed by float) at the given XPath, 
+    """ Add a vector (composed by float) at the given XPath,
     if the node does not exist, it will be created. Values will be
     overwritten if paths exists.
 
@@ -374,10 +399,10 @@ def add_float_vector(tixi, xpath, vector):
 
     # Get the field name and the parent CPACS path
     xpath_child_name = xpath.split("/")[-1]
-    xpath_parent = xpath[:-(len(xpath_child_name)+1)]
+    xpath_parent = xpath[:-(len(xpath_child_name) + 1)]
 
     if not tixi.checkElement(xpath_parent):
-        create_branch(tixi,xpath_parent)
+        create_branch(tixi, xpath_parent)
 
     vector = [float(v) for v in vector]
 
@@ -385,7 +410,7 @@ def add_float_vector(tixi, xpath, vector):
         tixi.updateFloatVector(xpath, vector, len(vector), format='%g')
         tixi.addTextAttribute(xpath, 'mapType', 'vector')
     else:
-        tixi.addFloatVector(xpath_parent, xpath_child_name, vector, \
+        tixi.addFloatVector(xpath_parent, xpath_child_name, vector,
                             len(vector), format='%g')
         tixi.addTextAttribute(xpath, 'mapType', 'vector')
 
@@ -402,24 +427,25 @@ def add_string_vector(tixi, xpath, vector):
         xpath (str): XPath of the vector to add
         vector (list): Vector of string to add
     """
-    
+
     # Strip trailing '/' (has no meaning here)
     if xpath.endswith('/'):
         xpath = xpath[:-1]
 
     # Get the field name and the parent CPACS path
     xpath_child_name = xpath.split("/")[-1]
-    xpath_parent = xpath[:-(len(xpath_child_name)+1)]
+    xpath_parent = xpath[:-(len(xpath_child_name) + 1)]
 
     vector_str = ";".join([str(elem) for elem in vector])
 
     if not tixi.checkElement(xpath_parent):
-        create_branch(tixi,xpath_parent)
+        create_branch(tixi, xpath_parent)
 
     if tixi.checkElement(xpath):
         tixi.updateTextElement(xpath, vector_str)
     else:
-        tixi.addTextElement(xpath_parent,xpath_child_name,vector_str)
+        tixi.addTextElement(xpath_parent, xpath_child_name, vector_str)
+
 
 def get_string_vector(tixi, xpath):
     """ Get a vector (of string) at given CPACS xpath
@@ -447,7 +473,8 @@ def get_string_vector(tixi, xpath):
 
     return string_vector
 
-def get_xpath_parent(xpath,level=1):
+
+def get_xpath_parent(xpath, level=1):
     """ Get the parent xpath at any level, 1 is parent just above the input xpath.
 
     Args:
@@ -461,15 +488,15 @@ def get_xpath_parent(xpath,level=1):
     if not xpath.startswith('/'):
         raise ValueError('"get_xpath_parent" must recieve an xpath as argument!')
 
-    if len(xpath.split('/'))-1 <= level:
+    if len(xpath.split('/')) - 1 <= level:
         raise ValueError('No parent available at this level')
 
     return '/'.join(xpath.split('/')[:-level])
 
 
 def create_branch(tixi, xpath, add_child=False):
-    """ Create a branch in the tixi handle and also all the missing parent nodes. 
-    Be careful, the xpath must be unique until the last element, it means, 
+    """ Create a branch in the tixi handle and also all the missing parent nodes.
+    Be careful, the xpath must be unique until the last element, it means,
     if several element exist, its index must be precised (index start at 1).
     e.g.: '/cpacs/vehicles/aircraft/model/wings/wing[2]/name'
 
@@ -493,15 +520,14 @@ def create_branch(tixi, xpath, add_child=False):
     xpath_split = xpath.split("/")
     xpath_count = len(xpath_split)
 
-    for i in range(xpath_count-1):
+    for i in range(xpath_count - 1):
         xpath_index = i + 2
         xpath_partial = '/'.join(str(m) for m in xpath_split[0:xpath_index])
-        xpath_parent = '/'.join(str(m) for m in xpath_split[0:xpath_index-1])
-        child = xpath_split[(xpath_index-1)]
+        xpath_parent = '/'.join(str(m) for m in xpath_split[0:xpath_index - 1])
+        child = xpath_split[(xpath_index - 1)]
         if tixi.checkElement(xpath_partial):
             if child == xpath_split[-1] and add_child:
                 namedchild_nb = tixi.getNamedChildrenCount(xpath_parent, child)
-                tixi.createElementAtIndex (xpath_parent,child,namedchild_nb+1)
+                tixi.createElementAtIndex(xpath_parent, child, namedchild_nb + 1)
         else:
             tixi.createElement(xpath_parent, child)
-            
\ No newline at end of file
