@@ -1,34 +1,38 @@
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
+"""
+!/usr/bin/env python3
+-*- coding: utf-8 -*-
 
-# ----------------------------------------------------------------------
-# Copyright 2021 CFS Engineering
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#     http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-# ----------------------------------------------------------------------
+----------------------------------------------------------------------
+Copyright 2021 CFS Engineering
 
-# Author: Aidan Jungo
+Licensed under the Apache License, Version 2.0 (the "License");
+you may not use this file except in compliance with the License.
+You may obtain a copy of the License at
+
+    http://www.apache.org/licenses/LICENSE-2.0
+
+Unless required by applicable law or agreed to in writing, software
+distributed under the License is distributed on an "AS IS" BASIS,
+WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+See the License for the specific language governing permissions and
+limitations under the License.
+----------------------------------------------------------------------
+
+Author: Aidan Jungo
+
+"""
 
 import os
 import pandas as pd
 
 from cpacspy.aeromap import AeroMap
 from cpacspy.aircraft import Aircraft
-from cpacspy.cpacsfunctions import open_tigl, open_tixi
+from cpacspy.cpacsfunctions import open_tigl, open_tixi, get_xpath_parent
 from cpacspy.utils import AEROPERFORMANCE_XPATH
 
 
 class CPACS:
+    """ CPACS class """
 
     def __init__(self, cpacs_file):
 
@@ -43,18 +47,19 @@ class CPACS:
             self.ac_name = self.tixi.getTextElement(ac_name_xpath)
 
         # Aircraft data
-        self.aircraft = Aircraft(self.tixi,self.tigl)
+        self.aircraft = Aircraft(self.tixi, self.tigl)
 
-        # AeroMaps
-        self.aeromaps = []
-        self.nb_aeromaps = 0
+        # Load aeroMaps
         self.load_all_aeromaps()
 
     def load_all_aeromaps(self):
         """ Load all the aeromaps present in the CPACS file as object. """
 
+        self.nb_aeromaps = 0
+        self.aeromaps = []
+
         for aeromap_uid in self.get_aeromap_uid_list():
-            aeromap = AeroMap(self.tixi,aeromap_uid)
+            aeromap = AeroMap(self.tixi, aeromap_uid)
             self.aeromaps.append(aeromap)
             self.nb_aeromaps += 1
 
@@ -69,7 +74,7 @@ class CPACS:
         aeromap_count = self.tixi.getNamedChildrenCount(AEROPERFORMANCE_XPATH, 'aeroMap')
         if aeromap_count:
             for i in range(aeromap_count):
-                aeromap_xpath = AEROPERFORMANCE_XPATH + '/aeroMap[' + str(i+1) + ']'
+                aeromap_xpath = AEROPERFORMANCE_XPATH + '/aeroMap[' + str(i + 1) + ']'
                 aeromap_uid = self.tixi.getTextAttribute(aeromap_xpath, 'uID')
                 uid_list.append(aeromap_uid)
         else:
@@ -80,38 +85,38 @@ class CPACS:
     def get_aeromap_by_uid(self, uid):
         """ Get an aeromap object by its uid. """
 
-        for am in self.aeromaps:
-            if am.uid == uid:
-                return am
+        for aeromap in self.aeromaps:
+            if aeromap.uid == uid:
+                return aeromap
 
         raise ValueError(f'No aeromap with "{uid}" as uid as been found!')
 
-    def create_aeromap(self,uid):
+    def create_aeromap(self, uid):
         """ Create a new aeromap object. """
 
         if ' ' in uid:
             raise ValueError('AeroMap uid should not contain any space!')
 
         if uid not in self.get_aeromap_uid_list():
-            new_aeromap = AeroMap(self.tixi,uid,create_new=True)
+            new_aeromap = AeroMap(self.tixi, uid, create_new=True)
             self.aeromaps.append(new_aeromap)
             self.nb_aeromaps += 1
             return new_aeromap
         else:
             raise ValueError('This uid already exit!')
 
-    def create_aeromap_from_csv(self,csv_path,uid=None):
+    def create_aeromap_from_csv(self, csv_path, uid=None):
         """ Create a new aeromap object from a CSV file. """
 
         if not uid:
-            head, tail = os.path.split(csv_path)
+            _, tail = os.path.split(csv_path)
             uid = tail.split('.')[0]
 
         if not os.path.exists(csv_path):
             raise ValueError(f'CSV file not found at {os.path.abspath(csv_path)}')
 
         new_aeromap = self.create_aeromap(uid)
-        new_aeromap.df = pd.read_csv(csv_path,keep_default_na=False)
+        new_aeromap.df = pd.read_csv(csv_path, keep_default_na=False)
 
         return new_aeromap
 
@@ -119,7 +124,7 @@ class CPACS:
         """ Duplicate an aeromap and retrun the new aeromap object. """
 
         # Check uid's
-        if not uid_base in self.get_aeromap_uid_list():
+        if uid_base not in self.get_aeromap_uid_list():
             raise ValueError('The AeroMap to duplicate does not exit!')
 
         if uid_duplicate in self.get_aeromap_uid_list():
@@ -127,7 +132,7 @@ class CPACS:
 
         # Get AeroMap and duplicate
         am_base = self.get_aeromap_by_uid(uid_base)
-        am_duplicated = AeroMap(self.tixi,uid_duplicate,create_new=True)
+        am_duplicated = AeroMap(self.tixi, uid_duplicate, create_new=True)
 
         # Copy data
         am_duplicated.df = am_base.df
@@ -138,7 +143,25 @@ class CPACS:
 
         return am_duplicated
 
-    def save_cpacs(self,cpacs_file,overwrite=False):
+    def delete_aeromap(self, uid):
+        """ Delete an aeromap from its uid. """
+
+        # Check if uid is valid
+        if ' ' in uid:
+            raise ValueError('AeroMap uid should not contain any space!')
+
+        if uid not in self.get_aeromap_uid_list():
+            raise ValueError(f'uid "{uid}"" does not exit! The aeroMap canno be deleted!')
+
+        # Remove the aeromap  from the CPACS file
+        aeromap = self.get_aeromap_by_uid(uid)
+        xpath = get_xpath_parent(aeromap.xpath, level=1)
+        self.tixi.removeElement(xpath)
+
+        # Reload the aeromaps to take into account the changes in the CPACS file
+        self.load_all_aeromaps()
+
+    def save_cpacs(self, cpacs_file, overwrite=False):
         """ Save a CPACS file from the TIXI object at a chosen path. """
 
         # Check for .xml file
@@ -150,7 +173,7 @@ class CPACS:
             find_name = False
             i = 1
             while not find_name:
-                cpacs_file_new_name = cpacs_file.split('.xml')[0] + f'_{str(i)}.xml' 
+                cpacs_file_new_name = cpacs_file.split('.xml')[0] + f'_{str(i)}.xml'
                 if not os.path.exists(cpacs_file_new_name):
                     find_name = True
                     cpacs_file = cpacs_file_new_name
@@ -159,10 +182,10 @@ class CPACS:
 
         self.tixi.save(cpacs_file)
 
-    def __str__(self): 
+    def __str__(self):
 
         text_line = []
-        text_line.append('\nCPACS file -----------------------------------------------------------------------------------------')
+        text_line.append('\nCPACS file ----------------------------------------------------------')
         text_line.append(' ')
         text_line.append(f'Aircraft name : {self.ac_name}')
         text_line.append(f'CPACS file path: {self.cpacs_file}')
@@ -171,7 +194,7 @@ class CPACS:
         for aeromap_uid in self.get_aeromap_uid_list():
             text_line.append('  ' + aeromap_uid)
         text_line.append(' ')
-        text_line.append('----------------------------------------------------------------------------------------------------\n')
+        text_line.append('---------------------------------------------------------------------\n')
 
         return ('\n').join(text_line)
 
